/******************************************************
 *                      CFileData.h
 *
 * Project : MergeKW                   Ver : 1.0
 * Programmer : Michael Taylor        Date : 7/3/99
 * Language : VC 5.0                Target : Win32
 * Purpose : This file provides the interface to the
 *           file data retrieved and stored in the registry.
 *****************************************************/
//@DOC

#if !defined(CFILEDATA_H_INCLUDED)
#define CFILEDATA_H_INCLUDED
#pragma once

#define MAKE_FILEDATA(x) ((CFileData*)(x))

/*@CLASS
 *   Simple helper class for the file information.
 *@BASE public | CObject
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   1/31/99  Original version
 */
class CFileData : public CObject
{
//@ACCESS Public Constructors and destructors
public:
   //@CMEMBER Constructor
   CFileData ( )
   { 
      m_szFileName = m_szDescript = TEXT("");
      m_bSelected = false;
   };
   
   //@CMEMBER Copy constructor
   CFileData ( const CFileData& Data )
   {  
      m_szFileName = Data.m_szFileName;
      m_szDescript = Data.m_szDescript;
      m_bSelected = Data.m_bSelected;
   };

   //@CMEMBER Destructor
   ~CFileData ( ) 
   {
      m_szFileName = m_szDescript = TEXT("");
   };

//@ACCESS Public Methods
public:
   //@CMEMBER Get the file name
   CString GetFileName ( void ) { return(m_szFileName); };

   //@CMEMBER Get the description
   CString GetDescription ( void ) { return(m_szDescript); };

   //@CMEMBER Get the state
   bool IsSelected ( void ) { return(m_bSelected); };

   //@CMEMBER Reads the file information from the registry
   bool Read ( const CString& szBaseKey, const CString& szFile );

   //@CMEMBER Set the file name
   void SetFileName ( const CString& szFile ) 
   { m_szFileName = szFile; };

   //@CMEMBER Set the description
   void SetDescript ( const CString& szDescript )
   { m_szDescript = szDescript; };

   //@CMEMBER Set the state
   void SetSelected ( bool bSet = true )
   { m_bSelected = bSet; };

   //@CMEMBER Writes the file's data to the registry
   bool Write ( const CString& szBaseKey );

//@ACCESS Private Data Members
private:
   //@CMEMBER File name
   CString m_szFileName; 

   //@CMEMBER Description
   CString m_szDescript; 

   //@CMEMBER Is it selected
   bool m_bSelected;  
};

/*@MFUNC 
 *   Reads the file data for the given file from the
 * registry.
 *@RDESC
 *   Returns TRUE on success, or FALSE otherwise.
 *@XREF
 *   <c CFileData>
 *@AUTHOR
 *   Michael Taylor
 *@HISTORY
 *   1/31/99 Original version
 */
inline bool CFileData::Read ( 
                     const CString& szKey,  //@PARM [IN] Key
                     const CString& szFile  //@PARM [IN] File
                     )
{
   CRegKey SubKey;
   DWORD dwSize, dwValue;
   CString szValue;
   CFileData* pData = NULL;
   bool bOK = true;

   //Open the key
   if (SubKey.Open(HKEY_CURRENT_USER, BuildPath(szKey, szFile)) == ERROR_SUCCESS)
   {
      //Get the path to the file
      dwSize = MAX_PATH;
      szValue = TEXT("");
      if (SubKey.QueryValue(szValue.GetBuffer(dwSize), PATH_VALUE, &dwSize) != ERROR_SUCCESS)
         bOK = false;

      szValue.ReleaseBuffer();
      szValue = BuildPath(szValue, szFile);
      m_szFileName = szValue;

      //Get the description
      dwSize = MAX_PATH;
      szValue = TEXT("");
      if (SubKey.QueryValue(szValue.GetBuffer(dwSize), DESCRIPT_VALUE, &dwSize) != ERROR_SUCCESS)
         bOK = false;

      szValue.ReleaseBuffer();
      m_szDescript = szValue;

      //Get the state
      dwValue = 0;
      if (SubKey.QueryValue(dwValue, SELECTED_VALUE) != ERROR_SUCCESS)
         bOK = false;

      m_bSelected = (dwValue != 0);

      //Clean up
      SubKey.Close();
   };

   return(bOK);
};

/*@MFUNC
 *   Writes a file data instance out to the registry.
 *@RDESC
 *   Returns TRUE if successful, or FALSE otherwise.
 *@XREF
 *   <c CFileData>
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   1/31/99  Original version
 */   
inline bool CFileData::Write (
          const CString& szBaseKey     //@PARM [IN] Base key
                      )
{  
   CString szKey, szFile = TEXT(""), szPath = TEXT("");
   CRegKey Key;   

   //Get the file and the path
   szPath = m_szFileName;
   int iPos = szPath.ReverseFind(TEXT('\\'));
   if (iPos != -1)
   {
      szFile = szPath.Mid(iPos + 1);
      szPath = szPath.Left(iPos);
   };

   //Create the new key
   szKey = BuildPath(szBaseKey, szFile);     
   if (Key.Create(HKEY_CURRENT_USER, szKey) != ERROR_SUCCESS)
      return(false);

   //Set the data
   if (Key.SetValue(szPath, PATH_VALUE) != ERROR_SUCCESS)
      return(false);
   if (Key.SetValue(m_szDescript, DESCRIPT_VALUE) != ERROR_SUCCESS)
      return(false);
   if (Key.SetValue(m_bSelected, SELECTED_VALUE) != ERROR_SUCCESS)
      return(false);

   //Cleanup   
   Key.Close();
   return(true);
};

/*@FUNC
 *   Reads the file list from the registry
 * and builds up the file list.
 *@RDESC
 *   Returns TRUE if successful, or FALSE otherwise.
 *@XREF
 *   <c CFileData>  <f WriteFileList>
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   7/3/99  Original version
 */   
inline bool ReadFileList (
             CObArray& FileList     //@PARM [OUT] File list
                         )
{
   CRegKey BaseKey;
   CString szKey;
   CFileData* pData;

   //Access the key where the files are stored
   CString szBaseKey = BuildPath(BASE_KEY, FILES_KEY);
   if (BaseKey.Open(HKEY_CURRENT_USER, szBaseKey) == ERROR_SUCCESS)
   {
      //Get each subkey(file) 
      DWORD dwSize;
      LONG lResult;
      DWORD dwEnumIndex = 0;
      do
      {
         dwSize = MAX_PATH;
         lResult = RegEnumKeyEx(BaseKey, dwEnumIndex++, szKey.GetBuffer(dwSize), &dwSize,
                                NULL, NULL, NULL, NULL);
         szKey.ReleaseBuffer();
         if (ERROR_SUCCESS == lResult)
         {
            //Create a new instance
            pData = new CFileData;  
            if (!pData)
               return(false);             

            if (!pData->Read(szBaseKey, szKey))
               delete pData;
            else
               FileList.Add(pData);
         };
      } while (ERROR_SUCCESS == lResult);
      BaseKey.Close();
   };

   return(true);
};

/*@FUNC
 *   Writes the file list specified to the registry.
 *@RDESC
 *   Returns TRUE if successful, or FALSE otherwise.
 *@XREF
 *   <c CFileData>  <f ReadFileList>
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   7/3/99  Original version
 */   
inline bool WriteFileList ( 
            const CObArray& FileList      //@PARM [IN] File list
                          )
{
   //Delete the old keys
   CleanReg();

   //Get the base key
   CString szKey = BuildPath(BASE_KEY, FILES_KEY);

   //For each file, write it
   CFileData* pData = NULL;
   DWORD dwSize = FileList.GetSize();
   for (DWORD dwIdx = 0;
        dwIdx < dwSize;
        dwIdx++)
   {
      pData = MAKE_FILEDATA(FileList[dwIdx]);      
      pData->Write(szKey);
   };

   return(true);
};

#endif  //Sentry